@@ -14,17 +14,6 @@
 # Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 # MA  02111-1307  USA
 
-# Copyrights (C)
-# for this R-port:
-#   1999 - 2008, Diethelm Wuertz, Rmetrics Foundation, GPL
-#   Diethelm Wuertz <wuertz@itp.phys.ethz.ch>
-#   www.rmetrics.org
-# for the code accessed (or partly included) from other R-ports:
-#   see R's copyright and license files
-# for the code accessed (or partly included) from contributed R-ports
-# and other sources
-#   see Rmetrics's copyright file
-
 
 ################################################################################
 # FUNCTIONS:            DESCRIPTION:
@@ -32,12 +21,13 @@
 #  pstable               Returns probabilities for stable DF
 #  qstable               Returns quantiles for stable DF
 #  rstable               Returns random variates for stable DF
+# UTILITY FUNCTION      DESCRIPTION:
 #  .integrateStable      Integrates internal functions for *stable
 ################################################################################
 
 
-dstable =
-function(x, alpha, beta, gamma = 1, delta = 0, pm = c(0, 1, 2))
+dstable <- 
+function(x, alpha, beta, gamma = 1, delta = 0, pm = c(0, 1, 2), log = FALSE)
 {
     # A function implemented by Diethelm Wuertz
 
@@ -66,6 +56,14 @@ function(x, alpha, beta, gamma = 1, delta = 0, pm = c(0, 1, 2))
 
     # FUNCTION:
 
+    # Parameters:
+    if (length(alpha) == 4) {
+       delta = alpha[4]
+       gamma = alpha[3]
+       beta = alpha[2]
+       alpha = alpha[1]
+    } 
+    
     # Settings:
     subdivisions = 1000
     tol = .Machine$double.eps
@@ -154,6 +152,9 @@ function(x, alpha, beta, gamma = 1, delta = 0, pm = c(0, 1, 2))
         cbind.data.frame(dist = "stable", alpha = alpha, beta = beta,
             gamma = gamma, delta = delta, pm = pm, row.names = "")
 
+    # Log:
+    if (log) ans = log(ans)
+    
     # Return Value:
     ans
 }
@@ -246,7 +247,7 @@ function(xarg, alpha, beta, tol, subdivisions)
 # ------------------------------------------------------------------------------
 
 
-pstable =
+pstable <- 
 function(q, alpha, beta, gamma = 1, delta = 0, pm = c(0, 1, 2))
 {
     # A function implemented by Diethelm Wuertz
@@ -430,7 +431,7 @@ function(xarg, alpha, beta, tol, subdivisions)
 # ------------------------------------------------------------------------------
 
 
-qstable =
+qstable <- 
 function(p, alpha, beta, gamma = 1, delta = 0, pm = c(0, 1, 2))
 {
     # A function implemented by Diethelm Wuertz
@@ -608,7 +609,7 @@ function(n, alpha, beta, gamma = 1, delta = 0, pm = c(0, 1, 2))
 
 
 .integrateStable =
-function (f, lower, upper, subdivisions, rel.tol, abs.tol, ...)
+function(f, lower, upper, subdivisions, rel.tol, abs.tol, ...)
 {
     # A function implemented by Diethelm Wuertz
 
